@@ -49,7 +49,7 @@ emfrail_control <- function(opt_fit = TRUE,
                             ca_test = TRUE,
                             only_ca_test = FALSE,
                             lik_ci = TRUE,
-                            lik_ci_intervals = list(interval = c(-2.5, 20),
+                            lik_ci_intervals = list(interval = c(-3, 20),
                                                     interval_stable = c(0, 20)),
                             nlm_control = list(stepmax = 1),
                             inner_control = list(eps = 0.0001,
